@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -66,6 +66,8 @@ static void dhcp_free(struct connman_dhcp *dhcp)
 	dhcp->nameservers = NULL;
 	dhcp->timeservers = NULL;
 	dhcp->pac = NULL;
+
+	g_free(dhcp);
 }
 
 /**
@@ -84,6 +86,7 @@ static void dhcp_invalidate(struct connman_dhcp *dhcp, bool callback)
 {
 	struct connman_service *service;
 	struct connman_ipconfig *ipconfig;
+	bool network_removed = false;
 	int i;
 
 	DBG("dhcp %p callback %u", dhcp, callback);
@@ -129,10 +132,17 @@ static void dhcp_invalidate(struct connman_dhcp *dhcp, bool callback)
 	__connman_ipconfig_set_gateway(ipconfig, NULL);
 	__connman_ipconfig_set_prefixlen(ipconfig, 0);
 
-	if (dhcp->callback && callback)
+	if (dhcp->callback && callback) {
+		g_hash_table_remove(network_table, dhcp->network);
+		network_removed = true;
 		dhcp->callback(dhcp->network, false, NULL);
+	}
 
 out:
+	if (!network_removed)
+		g_hash_table_remove(network_table, dhcp->network);
+
+	connman_network_unref(dhcp->network);
 	dhcp_free(dhcp);
 }
 
@@ -353,11 +363,9 @@ static void lease_available_cb(GDHCPClient *dhcp_client, gpointer user_data)
 
 	DBG("c_address %s", c_address);
 
-	if (address && c_address &&
-					g_strcmp0(address, c_address) != 0)
+	if (address && c_address && g_strcmp0(address, c_address) != 0)
 		ip_change = true;
-	else if (gateway && c_gateway &&
-					g_strcmp0(gateway, c_gateway) != 0)
+	else if (gateway && c_gateway && g_strcmp0(gateway, c_gateway) != 0)
 		ip_change = true;
 	else if (prefixlen != c_prefixlen)
 		ip_change = true;
@@ -584,18 +592,6 @@ static int dhcp_release(struct connman_dhcp *dhcp)
 	return 0;
 }
 
-static void remove_network(gpointer user_data)
-{
-	struct connman_dhcp *dhcp = user_data;
-
-	DBG("dhcp %p", dhcp);
-
-	dhcp_invalidate(dhcp, false);
-	dhcp_release(dhcp);
-
-	g_free(dhcp);
-}
-
 int __connman_dhcp_start(struct connman_network *network, dhcp_cb callback)
 {
 	struct connman_dhcp *dhcp;
@@ -618,13 +614,20 @@ int __connman_dhcp_start(struct connman_network *network, dhcp_cb callback)
 
 void __connman_dhcp_stop(struct connman_network *network)
 {
-	DBG("");
+	struct connman_dhcp *dhcp;
+
+	DBG("network_table %p network %p", network_table, network);
 
 	if (!network_table)
 		return;
 
-	if (g_hash_table_remove(network_table, network))
-		connman_network_unref(network);
+	dhcp = g_hash_table_lookup(network_table, network);
+	g_hash_table_remove(network_table, network);
+
+	if (dhcp) {
+		dhcp_release(dhcp);
+		dhcp_invalidate(dhcp, false);
+	}
 }
 
 int __connman_dhcp_init(void)
@@ -632,7 +635,7 @@ int __connman_dhcp_init(void)
 	DBG("");
 
 	network_table = g_hash_table_new_full(g_direct_hash, g_direct_equal,
-							NULL, remove_network);
+							NULL, NULL);
 
 	return 0;
 }
